/*
 * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file stdin.c functions that deal with reading from stdin */

#include <dirent.h> /* readdir() */
#include <assert.h>


#include "para.h"
#include "string.h"
#include "list.h"
#include "sched.h"
#include "fd.h"
#include "error.h"
#include "stdin.h"

/**
 * the pre_select function of the stdin task
 *
 * \param s the scheduler this task was registered to
 * \param t the task structure of the stdin task
 *
 * This function is always successful. If there is space left in the
 * buffer of the stdin task, it adds \p STDIN_FILENO to the read fd set
 * of \a s.
 */
static void stdin_pre_select(struct sched *s, struct task *t)
{
	struct stdin_task *sit = t->private_data;
	t->ret = 1;
	sit->check_fd = 0;
	if (sit->loaded >= sit->bufsize)
		return;
	sit->check_fd = 1;
	para_fd_set(STDIN_FILENO, &s->rfds, &s->max_fileno);
}

static void stdin_default_event_handler(struct task *t)
{
	PARA_NOTICE_LOG("%p: %s\n", t, para_strerror(-t->ret));
	unregister_task(t);
}

/**
 * the post select function of the stdin task
 *
 * \param s the scheduler this task was registered to
 * \param t the task structure of the stdin task
 *
 * This function checks if \p STDIN_FILENO was included by in the read fd set
 * of \a s during the previous pre_select call.  If yes, and \p STDIN_FILENO
 * appeears to be readable, data is read from stdin into the buffer of the
 * stdin task.
 */
static void stdin_post_select(struct sched *s, struct task *t)
{
	struct stdin_task *sit = t->private_data;
	ssize_t ret;

	t->ret = 1;
	if (!sit->check_fd)
		return;
	if (!FD_ISSET(STDIN_FILENO, &s->rfds))
		return;
	ret = read(STDIN_FILENO, sit->buf + sit->loaded, sit->bufsize - sit->loaded);
	if (ret < 0)
		t->ret = -E_STDIN_READ;
	else if (ret > 0) {
		sit->loaded += ret;
		t->ret = ret;
	} else
		t->ret = -E_STDIN_EOF;
	if (t->ret < 0)
		sit->error = t->ret;
}

/**
 * Initialize a stdin task structure with default values.
 *
 * \param sit The stdin task structure.
 *
 * This fills in the pre/post select function poinzters of the task structure
 * given by \a sit. It also sets up a default error handler which unregisters
 * the task. Moreover, \a loaded and \a eof are set to zero and \a bufsize is
 * initialized to 16 KB (but no buffer is allocated).
 */
void stdin_set_defaults(struct stdin_task *sit)
{
	sit->bufsize = 16 * 1024,
	sit->loaded = 0,
	sit->error = 0,
	sit->task.pre_select = stdin_pre_select;
	sit->task.post_select = stdin_post_select;
	sit->task.event_handler = stdin_default_event_handler;
	sit->task.private_data = sit;
	mark_fd_nonblocking(STDIN_FILENO);
	sprintf(sit->task.status, "stdin reader");
}
